﻿using System;
using System.Reflection;
using System.Windows;
using Infragistics.Controls.Charts;
using Microsoft.Xaml.Behaviors;

namespace xamDataChart_DragLineSeriesData.Behavior
{
    public class InteractiveLineSeries : Behavior<LineSeries>
    {
        protected LineSeries LineSeries { get { return this.AssociatedObject; } }

        public XamDataChart Chart
        {
            get { return (XamDataChart)GetValue(ChartProperty); }
            set { SetValue(ChartProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Chart.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ChartProperty =
            DependencyProperty.Register("Chart", typeof(XamDataChart), typeof(InteractiveLineSeries), new PropertyMetadata(null));

        protected override void OnAttached()
        {
            base.OnAttached();

            LineSeries.PreviewMouseLeftButtonDown += LineSeries_PreviewMouseLeftButtonDown;
            LineSeries.LayoutUpdated += LineSeries_LayoutUpdated;
        }

        void LineSeries_LayoutUpdated(object sender, EventArgs e)
        {
            // I use the LayoutUpdated event to assign some event handlers because I know the 
            // SeriesViewer should be valid at this point.
            if (LineSeries.SeriesViewer != null)
            {
                LineSeries.SeriesViewer.PreviewMouseMove += SeriesViewer_PreviewMouseMove;
                LineSeries.SeriesViewer.PreviewMouseUp += SeriesViewer_PreviewMouseUp;
                LineSeries.LayoutUpdated -= LineSeries_LayoutUpdated;
            }
        }

        void SeriesViewer_PreviewMouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            isDragging = false;

            draggingItem = null;
        }

        void SeriesViewer_PreviewMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (isDragging)
            {
                // Get the Y value of the mouse in the series.
                ScalerParams sparams = new ScalerParams(Chart.ActualWindowRect, Chart.ViewportRect, LineSeries.YAxis.IsInverted);
                double yValue = LineSeries.YAxis.GetUnscaledValue(e.GetPosition(LineSeries.RootCanvas).Y, sparams);

                // Adjust the value of the property.
                PropertyInfo pInfo = draggingItem.GetType().GetProperty(LineSeries.ValueMemberPath);
                pInfo.SetValue(draggingItem, yValue, null);
            }
        }

        bool isDragging = false;
        object draggingItem = null;

        void LineSeries_PreviewMouseLeftButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (Chart == null)
                return;

            // Get the point that was clicked on.
            var item = LineSeries.GetItemFromSeriesPixel(e.GetPosition(LineSeries.RootCanvas));
            if (item != null)
            {
                draggingItem = item;

                isDragging = true;
            }
        }
    }
}
