﻿using System.Collections.ObjectModel;
using System.ComponentModel;

namespace xamDataChart_DragLineSeriesData.Data
{
    public class EnergyProductionDataSample : EnergyProductionData
    {
        public EnergyProductionDataSample()
        {
            Add(new EnergyProduction { Region = "America", Country = "Canada", Coal = 400, Oil = 100, Gas = 175, Nuclear = 225, Hydro = 350 });
            Add(new EnergyProduction { Region = "Asia", Country = "China", Coal = 925, Oil = 200, Gas = 350, Nuclear = 400, Hydro = 625 });
            Add(new EnergyProduction { Region = "Europe", Country = "Russia", Coal = 550, Oil = 200, Gas = 250, Nuclear = 475, Hydro = 425 });
            Add(new EnergyProduction { Region = "Asia", Country = "Australia", Coal = 450, Oil = 100, Gas = 150, Nuclear = 175, Hydro = 350 });
            Add(new EnergyProduction { Region = "America", Country = "United States", Coal = 800, Oil = 250, Gas = 475, Nuclear = 575, Hydro = 750 });
            Add(new EnergyProduction { Region = "Europe", Country = "France", Coal = 375, Oil = 150, Gas = 350, Nuclear = 275, Hydro = 325 });
        }
    }

    public class EnergyProductionDataHistory : EnergyProductionData
    {
        public EnergyProductionDataHistory()
        {
            Add(new EnergyProduction { Year = "2000", Country = "Canada", Coal = 400, Oil = 100, Gas = 175, Nuclear = 225, Hydro = 350 });
            Add(new EnergyProduction { Year = "2010", Country = "Canada", Coal = 450, Oil = 150, Gas = 225, Nuclear = 255, Hydro = 400 });
            Add(new EnergyProduction { Year = "2020", Country = "Canada", Coal = 300, Oil = 200, Gas = 275, Nuclear = 325, Hydro = 450 });

            Add(new EnergyProduction { Year = "2000", Country = "China", Coal = 825, Oil = 200, Gas = 350, Nuclear = 400, Hydro = 625 });
            Add(new EnergyProduction { Year = "2010", Country = "China", Coal = 925, Oil = 250, Gas = 400, Nuclear = 420, Hydro = 685 });
            Add(new EnergyProduction { Year = "2020", Country = "China", Coal = 975, Oil = 300, Gas = 450, Nuclear = 450, Hydro = 725 });

            Add(new EnergyProduction { Year = "2000", Country = "Russia", Coal = 550, Oil = 200, Gas = 250, Nuclear = 475, Hydro = 425 });
            Add(new EnergyProduction { Year = "2010", Country = "Russia", Coal = 600, Oil = 220, Gas = 280, Nuclear = 525, Hydro = 485 });
            Add(new EnergyProduction { Year = "2020", Country = "Russia", Coal = 650, Oil = 250, Gas = 350, Nuclear = 575, Hydro = 525 });

            Add(new EnergyProduction { Year = "2000", Country = "Australia", Coal = 450, Oil = 100, Gas = 150, Nuclear = 175, Hydro = 350 });
            Add(new EnergyProduction { Year = "2010", Country = "Australia", Coal = 480, Oil = 120, Gas = 250, Nuclear = 225, Hydro = 400 });
            Add(new EnergyProduction { Year = "2020", Country = "Australia", Coal = 550, Oil = 180, Gas = 350, Nuclear = 275, Hydro = 450 });

            Add(new EnergyProduction { Year = "2000", Country = "United States", Coal = 800, Oil = 250, Gas = 475, Nuclear = 575, Hydro = 750 });
            Add(new EnergyProduction { Year = "2010", Country = "United States", Coal = 850, Oil = 300, Gas = 525, Nuclear = 625, Hydro = 800 });
            Add(new EnergyProduction { Year = "2020", Country = "United States", Coal = 900, Oil = 350, Gas = 575, Nuclear = 675, Hydro = 850 });

            Add(new EnergyProduction { Year = "2000", Country = "France", Coal = 375, Oil = 150, Gas = 350, Nuclear = 275, Hydro = 325 });
            Add(new EnergyProduction { Year = "2010", Country = "France", Coal = 425, Oil = 200, Gas = 400, Nuclear = 325, Hydro = 385 });
            Add(new EnergyProduction { Year = "2020", Country = "France", Coal = 455, Oil = 250, Gas = 450, Nuclear = 375, Hydro = 425 });
        }
    }
    public class EnergyProductionData : ObservableCollection<EnergyProduction>
    {

    }
    public class EnergyProduction : INotifyPropertyChanged
    {
        #region INotifyPropertyChanged Implementation
        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        protected void SetProperty<T>(ref T field, T value, string propertyName)
        {
            field = value;
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }
        #endregion

        public string Country { get; set; }
        public string Region { get; set; }
        public string Year { get; set; }

        // Non-Renewable Energy Sources
        private double _nuclear;
        public double Nuclear { get { return _nuclear; } set { SetProperty(ref _nuclear, value, "Nuclear"); UpdateValues(); } }

        private double _coal;
        public double Coal { get { return _coal; } set { SetProperty(ref _coal, value, "Coal"); UpdateValues(); } }

        private double _oil;
        public double Oil { get { return _oil; } set { SetProperty(ref _oil, value, "Oil"); UpdateValues(); } }

        private double _gas;
        public double Gas { get { return _gas; } set { SetProperty(ref _gas, value, "Gas"); UpdateValues(); } }
        // Renewable Energy Sources
        private double _hydro;
        public double Hydro { get { return _hydro; } set { SetProperty(ref _hydro, value, "Hydro"); UpdateValues(); } }

        private double _geoThermal;
        public double GeoThermal { get { return _geoThermal; } set { SetProperty(ref _geoThermal, value, "GeoThermal"); UpdateValues(); } }

        private double _solar;
        public double Solar { get { return _solar; } set { SetProperty(ref _solar, value, "Solar"); UpdateValues(); } }

        private double _wind;
        public double Wind { get { return _wind; } set { SetProperty(ref _wind, value, "Wind"); UpdateValues(); } }

        public double Total { get; set; }
        public double Renewable { get; set; }
        public double NonRenewable { get; set; }

        public void UpdateValues()
        {
            Renewable = Hydro + GeoThermal + Wind + Solar;
            NonRenewable = Coal + Oil + Gas + Nuclear;
            Total = Renewable + NonRenewable;
        }
    }

}